@@ -8,7 +8,6 @@ from torchvision.transforms import functional as F
 
 from ..core.transforms_interface import BasicTransform
 
-
 __all__ = ["ToTensorV2"]
 
 
@@ -77,7 +76,7 @@ class ToTensorV2(BasicTransform):
 
     @property
     def targets(self):
-        return {"image": self.apply, "mask": self.apply_to_mask}
+        return {"image": self.apply, "mask": self.apply_to_mask, "masks": self.apply_to_masks}
 
     def apply(self, img, **params):  # skipcq: PYL-W0613
         if len(img.shape) not in [2, 3]:
@@ -93,6 +92,9 @@ class ToTensorV2(BasicTransform):
             mask = mask.transpose(2, 0, 1)
         return torch.from_numpy(mask)
 
+    def apply_to_masks(self, masks, **params):
+        return [self.apply_to_mask(mask, **params) for mask in masks]
+
     def get_transform_init_args_names(self):
         return ("transpose_mask",)
 
